@@ -1,6 +1,6 @@
 from django import forms
 from django_th.models import ServicesActivated
-from django.utils.translation import ugettext as _
+from django.utils.translation import ugettext_lazy as _
 
 
 class ServiceChoiceForm(forms.Form):
@@ -8,6 +8,10 @@ class ServiceChoiceForm(forms.Form):
     def activated_services(self, provider=None):
         """
             get the activated services added from the administrator
+            :param provider: the selected provider
+            :type provider: string
+            :return: list of activated services
+            :rtype: list
         """
         services = ServicesActivated.objects.filter(status=1)
 
@@ -18,23 +22,28 @@ class ServiceChoiceForm(forms.Form):
             services = services.exclude(name__exact=provider)
 
         for class_name in services:
-            datas = (class_name, class_name.name.rsplit('Service', 1)[1])
+            datas = (class_name.name, class_name.name.rsplit('Service', 1)[1])
             choices.append(datas)
 
         return choices
 
 
 class ProviderForm(ServiceChoiceForm):
-
+    """
+        Set some HTML class to the Provider form
+    """
     provider = forms.ChoiceField()
 
     def __init__(self, *args, **kwargs):
         super(ProviderForm, self).__init__(*args, **kwargs)
         self.fields['provider'].choices = self.activated_services()
+        self.fields['provider'].widget.attrs['class'] = 'form-control'
 
 
 class ConsumerForm(ServiceChoiceForm):
-
+    """
+        Set some HTML class to the Consumer form
+    """
     consumer = forms.ChoiceField()
 
     def __init__(self, *args, **kwargs):
@@ -43,10 +52,13 @@ class ConsumerForm(ServiceChoiceForm):
         # the provider form
         self.fields['consumer'].choices = self.activated_services(
             self.initial['provider'])
+        self.fields['consumer'].widget.attrs['class'] = 'form-control'
 
 
 class ServicesDescriptionForm(forms.Form):
-
+    """
+        Set some HTML class to the Service form
+    """
     description = forms.CharField(
         widget=forms.TextInput(attrs={'placeholder':
                                       _('A description for your new service')})
