@@ -1,18 +1,20 @@
 # -*- coding: utf-8 -*-
 from __future__ import unicode_literals
-from django.shortcuts import render_to_response, render, redirect, get_object_or_404
+from django.shortcuts import render_to_response, render, redirect
+from django.shortcuts import get_object_or_404
 from django.contrib.auth.decorators import login_required
 from django.utils.decorators import method_decorator
 from django.http import HttpResponseRedirect
 from django.core.urlresolvers import reverse_lazy, reverse
-from django.views.generic import CreateView, DeleteView, ListView, TemplateView, UpdateView
+from django.views.generic import CreateView, DeleteView, ListView
+from django.views.generic import TemplateView, UpdateView
 from django.db.models import Q
-
+from django.utils.translation import ugettext as _
 from django.contrib.formtools.wizard.views import SessionWizardView
 
 # trigger_happy
 from django_th.models import TriggerService, UserService, ServicesActivated
-from django_th.forms.base import UserServiceForm
+from django_th.forms.base import UserServiceForm, TriggerServiceForm
 from django_th.forms.wizard import ConsumerForm
 
 from django_th.services import default_provider
@@ -34,6 +36,15 @@ import importlib
 
 
 def class_for_name(module_name, class_name):
+    """
+        Import a class dynamically
+        :param module_name: the name of a module
+        :param class_name: the name of a class
+        :type module_name: string
+        :type class_name: string
+        :return: Return the value of the named attribute of object.
+        :rtype: object
+    """
     # load the module, will raise ImportError if module cannot be loaded
     m = importlib.import_module(module_name)
     # get the class, will raise AttributeError if class cannot be found
@@ -44,6 +55,17 @@ def class_for_name(module_name, class_name):
 def get_service(service, model_form='models', form_name=''):
     """
         get the service name then load the model
+        :param service: the service name
+        :param model_form: could be 'models' or 'forms'
+        :param form_name: the name of the form is model_form is 'forms'
+        :type service: string
+        :type model_form: string
+        :type form_name: string
+        :return: the object of the spotted Class.
+        :rtype: object
+
+        :Example:
+
         class_name could be :
             th_rss.models
             th_rss.forms
@@ -77,21 +99,54 @@ def logout_view(request):
 
 def trigger_on_off(request, trigger_id):
     """
-        switch the status of the trigger then go back home
+        enable/disable the status of the trigger then go back home
+        :param trigger_id: the trigger ID to switch the status to True or False
+        :type trigger_id: int
     """
     trigger = get_object_or_404(TriggerService, pk=trigger_id)
     if trigger.status:
+        title = 'disabled'
+        title_trigger = _('Set this trigger on')
+        btn = 'success'
         trigger.status = False
     else:
+        title = _('Edit your service')
+        title_trigger = _('Set this trigger off')
+        btn = 'primary'
         trigger.status = True
     trigger.save()
 
-    return HttpResponseRedirect(reverse('base'))
+    return render(request, 'triggers/trigger_line.html',
+                           {'trigger': trigger,
+                            'title': title,
+                            'title_trigger': title_trigger,
+                            'btn': btn})
+
+
+def service_related_triggers_switch_to(request, user_service_id, switch):
+    """
+        switch the status of all the triggers related to the service, then go back home
+        :param service_id: the service ID to switch the status to True or False of all the related trigger
+        :type service_id: int
+        :param switch: the switch value
+        :type switch: string off or on
+    """
+    status = True
+    if switch == 'off':
+        status = False
+
+    TriggerService.objects.filter(provider__id=user_service_id).update(status=status)
+    TriggerService.objects.filter(consumer__id=user_service_id).update(status=status)
+
+    return HttpResponseRedirect(reverse('user_services'))
+
 
 
 def trigger_switch_all_to(request, switch):
     """
         switch the status of all the triggers then go back home
+        :param switch: the switch value
+        :type switch: string off or on
     """
     status = True
     if switch == 'off':
@@ -104,13 +159,6 @@ def trigger_switch_all_to(request, switch):
     return HttpResponseRedirect(reverse('base'))
 
 
-def qty_services_activated(user):
-    """
-        get the quantity of activated services
-    """
-    return UserService.objects.filter(user=user)
-
-
 def list_services(request, step):
     """
         get the activated services added from the administrator
@@ -131,73 +179,80 @@ def list_services(request, step):
 def renew_service(request, pk):
     """
         renew an existing service
+        :param pk: the primary key of the service to renew
+        :type pk: int
     """
     service = get_object_or_404(ServicesActivated, pk=pk)
     service_name = str(service.name)
     service_object = default_provider.get_service(service_name)
     lets_auth = getattr(service_object, 'auth')
-    #lets_callback = getattr(service_object, 'auth')
-    # print lets_callback
     return redirect(lets_auth(request))
-    # return redirect('base')
+
 
 #*************************************
 #  Part I : the Triggers
 #*************************************
 
 
-def trigger_edit_provider(request, trigger_id):
+def trigger_edit(request, trigger_id, edit_what):
     """
         edit the provider
+        :param trigger_id: ID of the trigger to edit
+        :param edit_what: edit a 'Provider' or 'Consumer' ?
+        :type trigger_id: int
+        :type edit_what: string
     """
+    if edit_what not in ('Provider', 'Consumer'):
+        #bad request
+        return redirect('base')
+
+    # get the trigger object
     service = TriggerService.objects.get(id=trigger_id)
-    service_name = str(service.provider.name.name).split('Service')[1]
-    model = get_service(service.provider.name.name)
+
+    if edit_what == 'Consumer':
+        # get the service name
+        service_name = str(service.consumer.name.name).split('Service')[1]
+        # get the model of this service
+        model = get_service(service.consumer.name.name)
+    else:
+        # get the service name
+        service_name = str(service.provider.name.name).split('Service')[1]
+        # get the model of this service
+        model = get_service(service.provider.name.name)
+
+    # get the data of this service linked to that trigger
     data = model.objects.get(trigger_id=trigger_id)
 
+    template_name = service_name.lower() + '/edit_' + edit_what.lower() + ".html"
+
     if request.method == 'POST':
-        form = get_service(service.provider.name.name, 'forms', 'ProviderForm')(
-            request.POST, instance=data)
+        if edit_what == 'Consumer':
+            form = get_service(
+                service.consumer.name.name, 'forms', edit_what + 'Form')(
+                request.POST, instance=data)
+        else:
+            form = get_service(
+                service.provider.name.name, 'forms', edit_what + 'Form')(
+                request.POST, instance=data)
+
         if form.is_valid():
             form.save()
             return HttpResponseRedirect(reverse('trigger_edit_thanks'))
     else:
-        form = get_service(
-            service.provider.name.name, 'forms', 'ProviderForm')(instance=data)
-    context = {'description': service.description}
-    return render(request, service_name.lower() + '/edit_provider.html', {
-        'form': form,
-        'context': context
-    })
+        if edit_what == 'Consumer':
+            form = get_service(
+                service.consumer.name.name, 'forms', edit_what + 'Form')(
+                instance=data)
+        else:
+            form = get_service(
+                service.provider.name.name, 'forms', edit_what + 'Form')(
+                instance=data)
 
-
-def trigger_edit_consumer(request, trigger_id):
-    """
-        edit the consumer
-    """
-    service = TriggerService.objects.get(id=trigger_id)
-    service_name = str(service.consumer.name.name).split('Service')[1]
-    model = get_service(service.consumer.name.name)
-    data = model.objects.get(trigger_id=trigger_id)
-
-    if request.method == 'POST':
-        form = get_service(service.consumer.name.name, 'forms', 'ConsumerForm')(
-            request.POST, instance=data)
-        if form.is_valid():
-            form.save()
-            return HttpResponseRedirect(reverse('trigger_edit_thanks'))
-    else:
-        form = get_service(
-            service.consumer.name.name, 'forms', 'ConsumerForm')(instance=data)
-    context = {'description': service.description}
-    return render(request, service_name.lower() + '/edit_consumer.html', {
-        'form': form,
-        'context': context
-    })
+    context = {'description': service.description, 'edit_what': edit_what}
+    return render(request, template_name, {'form': form, 'context': context})
 
 
 class TriggerListView(ListView):
-
     """
         list of Triggers
         the list can be filtered by service
@@ -205,35 +260,50 @@ class TriggerListView(ListView):
     context_object_name = "triggers_list"
     queryset = TriggerService.objects.all()
     template_name = "home.html"
-    paginate_by = 7
+    paginate_by = 3
 
     def get_queryset(self):
-        trigger_filter_by = None
+        trigger_filtered_by = None
+        # by default, sort by date_created
+        trigger_ordered_by = (str('-date_created'), )
         # get the Trigger of the connected user
         if self.request.user.is_authenticated():
             # if the user selected a filter, get its ID
-            if 'trigger_filter_by' in self.kwargs:
+            if 'trigger_filtered_by' in self.kwargs:
                 user_service = UserService.objects.filter(
-                    user=self.request.user, name=self.kwargs['trigger_filter_by'])
-                trigger_filter_by = user_service[0].id
+                    user=self.request.user,
+                    name=self.kwargs['trigger_filtered_by'])
+                trigger_filtered_by = user_service[0].id
 
-            # no filter selected : display all
-            if trigger_filter_by is None:
-                return self.queryset.filter(user=self.request.user).order_by('-date_created')
-            # filter selected : display all related trigger
+            if 'trigger_ordered_by' in self.kwargs:
+                """
+                    sort by 'name' property in the related model UserService
+                """
+                order_by = str(self.kwargs['trigger_ordered_by'] + "__name")
+                # append to the tuple, the selected 'trigger_ordered_by'
+                # choosen in the dropdown
+                trigger_ordered_by = (order_by, ) + trigger_ordered_by
+
+            # no filter selected
+            if trigger_filtered_by is None:
+                return self.queryset.filter(user=self.request.user).order_by(*trigger_ordered_by).select_related('consumer__name', 'provider__name')
+
+            # filter selected : display all related triggers
             else:
                 # here the queryset will do :
                 # 1) get trigger of the connected user AND
                 # 2) get the triggers where the provider OR the consumer match
                 # the selected service
-                return self.queryset.filter(user=self.request.user).filter(Q(provider=trigger_filter_by) | Q(consumer=trigger_filter_by)).order_by('-date_created')
+                return self.queryset.filter(user=self.request.user).filter(
+                    Q(provider=trigger_filtered_by) |
+                    Q(consumer=trigger_filtered_by)).order_by(*trigger_ordered_by).select_related('consumer__name', 'provider__name')
         # otherwise return nothing when user is not connected
         return TriggerService.objects.none()
 
     def get_context_data(self, **kw):
         """
             get the data of the view
-            
+
             data are :
             1) number of triggers enabled
             2) number of triggers disabled
@@ -241,42 +311,45 @@ class TriggerListView(ListView):
             4) list of activated services by the connected user
         """
         triggers_enabled = triggers_disabled = services_activated = ()
+
+        context = super(TriggerListView, self).get_context_data(**kw)
         if self.request.user.is_authenticated():
             # get the enabled triggers
             triggers_enabled = TriggerService.objects.filter(
-                user=self.request.user, status=1)
+                user=self.request.user, status=1).count()
             # get the disabled triggers
             triggers_disabled = TriggerService.objects.filter(
-                user=self.request.user, status=0)
+                user=self.request.user, status=0).count()
             # get the activated services
-            services_activated = qty_services_activated(self.request.user)
-        context = super(TriggerListView, self).get_context_data(**kw)
+            user_service = UserService.objects.filter(user=self.request.user)
+            """
+                List of triggers activated by the user
+            """
+            context['trigger_filter_by'] = user_service
+            """
+                number of service activated for the current user
+            """
+            services_activated = user_service.count()
+
         """
             which triggers are enabled/disabled
         """
-        context['nb_triggers'] = {'enabled': len(triggers_enabled),
-                                  'disabled': len(triggers_disabled)}
+        context['nb_triggers'] = {'enabled': triggers_enabled,
+                                  'disabled': triggers_disabled}
         """
             Number of services activated
         """
-        context['nb_services'] = len(services_activated)
-        """
-            List of triggers activated by the user
-        """
-        if self.request.user.is_authenticated():
-            context['trigger_filter_by'] = UserService.objects.filter(
-                user=self.request.user)
+        context['nb_services'] = services_activated
 
         return context
 
 
 class TriggerUpdateView(UpdateView):
-
     """
         Form to update description
     """
-    queryset = TriggerService.objects.all()
-    fields = ['description']
+    model = TriggerService
+    form_class = TriggerServiceForm
     template_name = "triggers/edit_description_trigger.html"
     success_url = reverse_lazy("trigger_edit_thanks")
 
@@ -286,7 +359,6 @@ class TriggerUpdateView(UpdateView):
 
 
 class TriggerEditedTemplateView(TemplateView):
-
     """
         just a simple form to say thanks :P
     """
@@ -299,21 +371,19 @@ class TriggerEditedTemplateView(TemplateView):
 
 
 class TriggerDeleteView(DeleteView):
-
     """
         page to delete a trigger
     """
-    queryset = TriggerService.objects.all()
+    model = TriggerService
     template_name = "triggers/delete_trigger.html"
     success_url = reverse_lazy("trigger_delete_thanks")
 
-    @method_decorator(login_required)
+    # @method_decorator(login_required)
     def dispatch(self, *args, **kwargs):
         return super(TriggerDeleteView, self).dispatch(*args, **kwargs)
 
 
 class TriggerDeletedTemplateView(TemplateView):
-
     """
         just a simple form to say thanks :P
     """
@@ -330,7 +400,6 @@ class TriggerDeletedTemplateView(TemplateView):
 #  Part II : the UserServices
 #*************************************
 class UserServiceListView(ListView):
-
     """
         List of the services activated by the user
     """
@@ -347,7 +416,6 @@ class UserServiceListView(ListView):
 
     def get_context_data(self, **kw):
         context = super(UserServiceListView, self).get_context_data(**kw)
-        services_activated = ()
         if self.request.user.is_authenticated():
             activated_qs = ServicesActivated.objects.all()
             service_list_available = UserService.objects.filter(
@@ -362,17 +430,15 @@ class UserServiceListView(ListView):
             context['service_list_available'] = service_list_available
 
             # get the activated services
-            services_activated = qty_services_activated(self.request.user)
             """
                 Number of services activated
             """
-            context['nb_services'] = len(services_activated)
+            context['nb_services'] = nb_user_service
 
         return context
 
 
 class UserServiceCreateView(CreateView):
-
     """
         Form to add a service
     """
@@ -403,11 +469,6 @@ class UserServiceCreateView(CreateView):
 
         return HttpResponseRedirect(reverse('service_add_thanks'))
 
-    def get_context_data(self, **kw):
-        context = super(UserServiceCreateView, self).get_context_data(**kw)
-        context['action'] = 'add_service'
-        return context
-
     def get_form_kwargs(self, **kwargs):
         kwargs = super(UserServiceCreateView, self).get_form_kwargs(**kwargs)
         kwargs['initial']['user'] = self.request.user
@@ -415,7 +476,6 @@ class UserServiceCreateView(CreateView):
 
 
 class UserServiceRenewTemplateView(TemplateView):
-
     """
         page to renew a service
         usefull when revoking has been done or made changes
@@ -430,11 +490,10 @@ class UserServiceRenewTemplateView(TemplateView):
 
 
 class UserServiceDeleteView(DeleteView):
-
     """
         page to delete a service
     """
-    queryset = UserService.objects.all()
+    model = UserService
     template_name = "services/delete_service.html"
     success_url = reverse_lazy("service_delete_thanks")
 
@@ -444,7 +503,6 @@ class UserServiceDeleteView(DeleteView):
 
 
 class UserServiceAddedTemplateView(TemplateView):
-
     """
         just a simple form to say thanks :P
     """
@@ -458,7 +516,6 @@ class UserServiceAddedTemplateView(TemplateView):
 
 
 class UserServiceDeletedTemplateView(TemplateView):
-
     """
         just a simple form to say thanks :P
     """
@@ -553,13 +610,13 @@ class UserServiceWizard(SessionWizardView):
             if i == 0:
                 trigger_provider = UserService.objects.get(
                     name=data['provider'],
-                    user=self.request.user)
+                    user=self.request.user.id)
                 model_provider = get_service(data['provider'], 'models')
             # get the service we selected at step 2 : consumer
             elif i == 2:
                 trigger_consumer = UserService.objects.get(
                     name=data['consumer'],
-                    user=self.request.user)
+                    user=self.request.user.id)
                 model_consumer = get_service(data['consumer'], 'models')
             # get the description we gave for the trigger
             elif i == 4:
